/******************************************************************************
 * Copyright (C) 2003 - Nelson Posse Lago
 *
 * This file is part of DLADSPA, a system for distributed processing
 * of LADSPA plugins.
 *
 * DLADSPA is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * DLADSPA is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * or just check their website at http://www.fsf.org
 ******************************************************************************/


#include <dladspa/container.h>
#include <dladspa/constants.h>

#include <dlowlat/masterslave/masterslave.h>

#include <dladspa/find_plugin.h>

using namespace DLowLat;

namespace DLADSPA {


Container::Container (Slave& processor, const char* spec,
                        unsigned long sampleRate) : pluginType (spec)
{
    descriptor = pluginType.descriptor ();
    plugin = descriptor->instantiate (descriptor, sampleRate);
    if (descriptor->activate) {
        descriptor->activate (plugin);
    }

    sampleCount = 0;
    constantsSize = 1;
    functionToCall = FUNCTION_RUN;

    theBlocks.push_back (processor.newDataBlock());
    theBlocks.back()->setBlock (&functionToCall, constantsSize, IN, false);

    theBlocks.push_back (processor.newDataBlock());
    theBlocks.back()->setBlock (&sampleCount, constantsSize, IN, false);

    setupPorts(processor);
};

Container::~Container ()
{
    if (descriptor->deactivate) {
        descriptor->deactivate (plugin);
    }

    if (descriptor->cleanup) {
        descriptor->cleanup (plugin);
    }

    for (unsigned int i = 0; i < descriptor->PortCount; ++i) {
        if (LADSPA_IS_PORT_CONTROL (descriptor->PortDescriptors[i])) {
            delete buffers[i];
        } else {
            delete[] buffers[i];
        }
    }
};

void
Container::process ()
{
    if (functionToCall == FUNCTION_RUN) {
        descriptor->run (plugin, sampleCount);
    }
};

void
Container::setupPorts (Slave& processor)
{
    // FIXME If the client sets up a single buffer as both an input
    // and an output port (for in-place processing), we don't realize
    // it; instead, we create independent buffers and miss the
    // opportunity of performing in-place processing here. No problem,
    // but things could be more efficient with in-place processing.
    for (unsigned int i = 0; i < descriptor->PortCount; ++i) {
        theBlocks.push_back (processor.newDataBlock());

        Direction dir = NONE;
        // FIXME!! Weind naming! On the proxy side this is reversed.
        isOut (dir, LADSPA_IS_PORT_OUTPUT(descriptor->PortDescriptors[i]));
        isIn (dir, LADSPA_IS_PORT_INPUT(descriptor->PortDescriptors[i]));

        if (LADSPA_IS_PORT_CONTROL (descriptor->PortDescriptors[i])) {
            buffers.push_back (new LADSPA_Data);
            theBlocks.back()->setBlock (buffers.back(), constantsSize, dir, false);
            descriptor->connect_port (plugin, i, buffers.back());
        } else {
            buffers.push_back (new LADSPA_Data[BUFFER_SIZE]);
            theBlocks.back()->setBlock (buffers.back(), sampleCount, dir, true);
            descriptor->connect_port (plugin, i, buffers.back());
        }
    }
};

} // namespace
