#ifndef DLADSPA_CONTAINER_H
#define DLADSPA_CONTAINER_H

/******************************************************************************
 * Copyright (C) 2003 - Nelson Posse Lago
 *
 * This file is part of DLADSPA, a system for distributed processing
 * of LADSPA plugins.
 *
 * DLADSPA is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * DLADSPA is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * or just check their website at http://www.fsf.org
 ******************************************************************************/


/******************************************************************************
 * This is the class that calls the actual LADSPA plugin to process the data
 * received from the remote end. It is responsible for creating the local
 * buffers that correspond to the remote ones, register them with the Slave
 * class that will perform communication with the Master and connect these
 * buffers to the ports in the LADSPA plugin. When data arrives from the
 * remote end, Slave calls the process() method in this class which, in turn,
 * calls process() on the LADSPA plugin.
 ******************************************************************************/

// Files that #include this one may also need to include:
// <dlowlat/masterslave/masterslave.h>
// <dladspa/constants.h>

#include <ladspa_sdk/ladspa.h>

#include <vector>

#include <dlowlat/masterslave/datablocks.h>

#include <dlowlat/masterslave/endianness.h>

#include <dladspa/plugin_chain.h>

// Needed forward declarations; maybe we should just #include the relevant
// files here...
namespace DLowLat {
    class Slave;
    class UdpWrite;
    class UdpRead;
}

namespace DLADSPA {

class Container {
public:
    Container (DLowLat::Slave& processor, const char* pluginSpec,
                                                    unsigned long sampleRate);
    ~Container();
    // this is the callback function that is called by the Slave processor
    void process();
private:
    // These need to be sent/received together with the rest of the LADSPA ports.
    size_t constantsSize, sampleCount;
    // Auxiliary method for the constructor
    void setupPorts(DLowLat::Slave&);
    // This is received from the remote end together with the data
    // from the LADSPA ports.
    unsigned int functionToCall;
    const LADSPA_Descriptor* descriptor;
    // Although we only use this on the constructor, this object needs
    // to exist as long as the descriptor exists; that's why we keep it here.
    const PluginChainType pluginType;
    LADSPA_Handle plugin;
    // These are the blocks we register to be sent/received by Slave
    std::vector<DLowLat::DataBlock> theBlocks;
    // These are the buffers kept "inside" the DataBlocks defined above
    // which are connected to the ports of the LADSPA plugin.
    std::vector<LADSPA_Data*> buffers;
};

} // namespace

#endif
