#ifndef DLADSPA_FUNCTION_POINTERS_H
#define DLADSPA_FUNCTION_POINTERS_H

/******************************************************************************
 * Copyright (C) 2003 - Nelson Posse Lago
 *
 * This file is part of DLADSPA, a system for distributed processing
 * of LADSPA plugins.
 *
 * DLADSPA is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * DLADSPA is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * or just check their website at http://www.fsf.org
 ******************************************************************************/


/******************************************************************************
 * LADSPA_Descriptors do their magic by keeping pointers to the various
 * functions that perform the activate, run and other tasks of the plugin
 * they describe. We have several classes that present themselves through
 * a LADSPA_Descriptor interface. Since we can't take C-like pointers to
 * member functions to put in the LADSPA_Descriptor, we use the templates
 * below instead. Since we're taking pointers to these functions, they
 * can't be inlined.
 ******************************************************************************/
 

#include <ladspa.h>

// FIXME These should be inside the DLADSPA namespace, but that triggers
// a bug in gcc 2.95

template <class T>
LADSPA_Handle
instantiate (const LADSPA_Descriptor* descriptor, unsigned long sampleRate)
{
    return new T (descriptor, sampleRate);
};

template <class T>
void
connect_port (LADSPA_Handle instance, unsigned long port, LADSPA_Data* ptr)
{
    static_cast<T*>(instance)->connect_port (port, ptr);
};

template <class T>
void
activate (LADSPA_Handle instance)
{
    static_cast<T*>(instance)->activate ();
};

template <class T>
void
run (LADSPA_Handle instance, unsigned long sampleCount)
{
    static_cast<T*>(instance)->run (sampleCount);
};
    
template <class T>
void
run_adding (LADSPA_Handle instance, unsigned long sampleCount)
{
    static_cast<T*>(instance)->run_adding (sampleCount);
};

template <class T>
void
set_run_adding_gain (LADSPA_Handle instance, LADSPA_Data gain)
{
    static_cast<T*>(instance)->set_run_adding_gain (gain);
};

template <class T>
void
deactivate (LADSPA_Handle instance)
{
    static_cast<T*>(instance)->deactivate ();
};

template <class T>
void
cleanup (LADSPA_Handle instance)
{
    delete static_cast<T*>(instance);
};

#endif // ifndef DLADSPA_FUNCTION_POINTERS_H
