/******************************************************************************
 * Copyright (C) 2003 - Nelson Posse Lago
 *
 * This file is part of DLADSPA, a system for distributed processing
 * of LADSPA plugins.
 *
 * DLADSPA is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * DLADSPA is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * or just check their website at http://www.fsf.org
 ******************************************************************************/


/******************************************************************************
 * This file is to be compiled as a dll that behaves like any other LADSPA dll:
 * when dlopen()'ed, the function ladspa_descriptor() may be used to get
 * information about available plugins, in the form of LADSPA_Descriptors. In
 * our case, there will be a single plugin, a MultiChannelPlugin that wraps
 * two instances of plugin 1048, mono amplifier (an example plugin from
 * the ladspa SDK).
 * To do this, this file instantiates a Loader object when loaded (the
 * "init" object), which creates the correct MultiChannelPluginType.
 ******************************************************************************/


#include <vector>

#include <ladspa.h>

extern "C" {
#include <ladspa_sdk/utils.h>
}

#include <dladspa/multi_channel_plugin.h>

using namespace DLADSPA;

// Let's hide the stuff that is not part of the LADSPA interface (that is,
// the ladspa_descriptor() function) in an unnamed namespace.

namespace {

/******************************************************************************
 *                           The Loader Class
 ******************************************************************************/
class Loader {
public:
    Loader ();
    inline const LADSPA_Descriptor* desc ();
private:
    const LADSPA_Descriptor *descriptor, *multiDescriptor;
    void* lib;
    MultiChannelPluginType* pluginType;
};

Loader::Loader ()
{
    // Load the actual plugin descriptor
    void* lib = loadLADSPAPluginLibrary ("/usr/lib/ladspa/amp.so");
    descriptor = findLADSPAPluginDescriptor (lib, "amp.so", "amp_mono");

    // Create the data needed by MultiChannelPluginType
    vector<PortOptions> controls, wetnesses;
    controls.push_back (PortOptions());
    controls[0].locked = false;
    controls[0].enabled = false;
    controls[0].values.push_back (3.0);
    controls[0].values.push_back (3.0);

    wetnesses.push_back (PortOptions());
    wetnesses[0].locked = true;
    wetnesses[0].enabled = false;
    wetnesses[0].values.push_back (1.0);
    wetnesses[0].values.push_back (1.0);

    // Create the "fake" descriptor
    pluginType = new MultiChannelPluginType (descriptor, 2, wetnesses, controls);
    multiDescriptor = pluginType->descriptor();
};

inline const LADSPA_Descriptor*
Loader::desc ()
{
    return multiDescriptor;
};


// All the work is done by the Loader constructor; isn't that pretty? ;-)
Loader init;

} // Close the unnamed namespace


// Clients will use this function to query which are the available plugins
extern "C"
{
    const LADSPA_Descriptor* ladspa_descriptor (unsigned long i)
    {
        if (i > 0)
            return 0;
        else
            return init.desc();
    }
} // extern "C"
