#ifndef DLOWLAT_ENDIANNESS_H
#define DLOWLAT_ENDIANNESS_H

/******************************************************************************
 * Copyright (C) 2003 - Nelson Posse Lago
 *
 * This file is part of DLADSPA, a system for distributed processing
 * of LADSPA plugins.
 *
 * DLADSPA is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * DLADSPA is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * or just check their website at http://www.fsf.org
 ******************************************************************************/


namespace DLowLat {

// A simple functor to check if some endianness is the same
// as the host's endianness. This is a singleton.

enum Endianness { BIGENDIAN = 0, LITTLEENDIAN = 1 };

class EndiannessChecker {
public:
    EndiannessChecker ();
    inline bool operator () (Endianness e);
    inline Endianness hostOrder ();
private:
    // These are static to save a little runtime speed
    // after the first time this functor is used; This
    // class is used in time-critical contexts.
    static Endianness theHostOrder;
    static bool initialized;
    void initialize();
};

inline bool
EndiannessChecker::operator() (Endianness e)
{
    return e == theHostOrder;
};

inline Endianness
EndiannessChecker::hostOrder ()
{
    return theHostOrder;
};

} // Namespace DLowLat

#endif // DLOWLAT_ENDIANNESS_H
