/******************************************************************************
 * Copyright (C) 2003 - Nelson Posse Lago
 *
 * This file is part of DLADSPA, a system for distributed processing
 * of LADSPA plugins.
 *
 * DLADSPA is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * DLADSPA is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * or just check their website at http://www.fsf.org
 ******************************************************************************/


using namespace std;

#include <dlowlat/transport/ip/tcp/sockets.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>

namespace DLowLat
{

Tcp::Tcp (Tcp& orig)
{
    connected = orig.connected;
    theDataSocket = dup (orig.theDataSocket);
};

Tcp::~Tcp ()
{
    close (theDataSocket);
};

TcpClient::TcpClient (IPAddress remote)
{
    theDataSocket = socket (PF_INET, SOCK_STREAM, 0);
    if (int i = connect (theDataSocket, remote.sockaddr(), remote.size()) != 0) {
        cerr << "sockets.cc, TcpClient: unable to connect" << endl;
        // FIXME
        exit (1);
    }
    connected = true;
};

TcpServer::TcpServer (IPAddress& local)
{
    theListenSocket = socket (PF_INET, SOCK_STREAM, 0);
    if (bind (theListenSocket, local.sockaddr(), local.size()) != 0) {
        cerr << "sockets.cc, TcpServer: cannot bind to ";
        cerr << local;
        cerr << endl;
        // FIXME!
        exit (1);
    // FIXME Magic number; we should really use poll()...
    // This queues up to 2 connections
    } else if (listen (theListenSocket, 2) != 0) {
        cerr << "sockets.cc, TcpServer: error on listen()" << endl;
        // FIXME!
        exit (1);
    };
};

IPAddress
Tcp::peerIP () const
{
    if (! isConnected()) {
        cerr << "sockets.cc, Tcp: peerIP called when not connected" << endl;
        // FIXME!!!
        exit (1);
    }
    IPAddress addr;
    socklen_t tmp = addr.size();
    getpeername (theDataSocket, addr.sockaddr(), &tmp);
    return addr;
};

IPAddress
Tcp::myIP () const
{
    IPAddress addr;
    socklen_t tmp = addr.size();
    getsockname (theDataSocket, addr.sockaddr(), &tmp);
    return addr;
};

void
Tcp::sendString (string& s)
{
    if (! connected || s.size() == 0)
        return;
    ostringstream tmp;
    tmp << s.size() << " " << s;
    // FIXME: Hopefully this will never fail ;-)
    write (theDataSocket, tmp.str().c_str(), tmp.str().size());
};

void
Tcp::readString (string& s)
{
    // We read a size, followed by a space, followed by a string of that size.
    if (! connected)
        return;

    char c = '\0';
    stringstream tmp;
    while (c != ' ' && read (theDataSocket, &c, 1) > 0) {
        tmp << c;
    }
    long size;
    tmp >> size;

    char buf [size +1];
    read (theDataSocket, buf, size);
    buf[size] = '\0';
    s += buf;
};

void
TcpServer::waitConnection ()
{
    IPAddress addr;
    socklen_t bogus = addr.size();
    theDataSocket = accept (theListenSocket, addr.sockaddr(), &bogus);
    if (theDataSocket > 1) {
        connected = true;
    } else if (theDataSocket < 0) {
        if (theDataSocket == EINTR) {
            // We received a signal; let's return with "connect"
            // set to false and let the client deal with it.
            return;
        } else {
            cerr << "TcpServer::waitConnection: Error waiting for connection" << endl;
            exit (0);
        }
    }
};

} // namespace
