#ifndef DLOWLAT_TCP_SOCKETS
#define DLOWLAT_TCP_SOCKETS

/******************************************************************************
 * Copyright (C) 2003 - Nelson Posse Lago
 *
 * This file is part of DLADSPA, a system for distributed processing
 * of LADSPA plugins.
 *
 * DLADSPA is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * DLADSPA is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * or just check their website at http://www.fsf.org
 ******************************************************************************/


/******************************************************************************
 * This is a simple class that allows the transmission and reception of
 * text strings through a TCP network channel, using standard "<<" and ">>"
 * operators.
 ******************************************************************************/

#include <dlowlat/transport/ip/ipaddress.h>
#include <string>
#include <typeinfo>

#include <unistd.h>
#include <sstream>
#include <iostream>

namespace DLowLat
{

class Tcp
{
public:
    Tcp () { connected = false; };
    Tcp (Tcp&);
    ~Tcp ();
    IPAddress peerIP () const;
    IPAddress myIP () const;
    void sendString (std::string& s);
    void readString (std::string& s);
    bool isConnected () const { return connected; };
protected:
    int theDataSocket;
    bool connected;
};

class TcpClient: public Tcp
{
public:
    TcpClient (IPAddress remote);
private:
};

class TcpServer: public Tcp
{
public:
    TcpServer (IPAddress& local);
    void waitConnection ();
private:
    void init ();
    int theListenSocket;
};


// These templates should allow for the transmission/reception of anything
// for which "operator << ()" and "operator >> ()" are defined.
template <typename T>
Tcp&
operator << (Tcp& channel, const T& val)
{
    std::ostringstream tmp;
    tmp << val;
    std::string s = tmp.str();
    channel.sendString (s);
    return channel;
};

template <typename T>
Tcp&
operator >> (Tcp& channel, T& val)
{
    std::string s;
    channel.readString (s);
    std::stringstream tmp;
    tmp << s;
    tmp >> val;
    return channel;
};

template <>
Tcp&
operator >> (Tcp& channel, std::string& val)
{
    channel.readString (val);
    return channel;
};


} // namespace

#endif // DLOWLAT_TCP_SOCKETS
