/******************************************************************************
 * Copyright (C) 2003 - Nelson Posse Lago
 *
 * This file is part of DLADSPA, a system for distributed processing
 * of LADSPA plugins.
 *
 * DLADSPA is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * DLADSPA is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * or just check their website at http://www.fsf.org
 ******************************************************************************/


using namespace std;

#include <dlowlat/transport/ip/ipaddress.h>
#include <dlowlat/transport/ip/udp/sockets.h>

// socket
#include <sys/types.h>
#include <sys/socket.h>

// close
#include <unistd.h>


namespace DLowLat {

/******************************************************************************
 * Constructors for the three classes; create and "connect" the socket
 ******************************************************************************/

Udp::Udp ()
{
    socket = ::socket (PF_INET, SOCK_DGRAM, 0);
};

Udp::Udp (Udp& orig)
{
    socket = dup (orig.socket);
};

IPAddress
Udp::address () const
{
    IPAddress addr;
    fillAddress (addr);
    return addr;
};

void
UdpWrite::fillAddress (IPAddress& addr) const
{
    socklen_t tmp = addr.size();
    getpeername (socket, addr.sockaddr(), &tmp);
};

void
UdpRead::fillAddress (IPAddress& addr) const
{
    socklen_t tmp = addr.size();
    getsockname (socket, addr.sockaddr(), &tmp);
};

UdpWrite::UdpWrite (IPAddress& address)
{
    connect (socket, address.sockaddr(), address.size());
    this->fillAddress (address);
};

UdpWrite::UdpWrite (UdpWrite& orig) : Udp (orig) { } ;

UdpRead::UdpRead (IPAddress& address, bool blocking)
{
    bind (socket, address.sockaddr(), address.size());

    // hack for blockingMode() to work on initialization
    block = ! blocking;
    blockingMode (blocking);
    this->fillAddress (address);
};

UdpRead::UdpRead (UdpRead& orig) : Udp (orig)
{
    block = orig.block;
};

void
UdpRead::flush ()
{
    bool oldBlockingMode = blockingMode();
    blockingMode (false);
    while (read (socket, 0, 65534) > 0);
    blockingMode (oldBlockingMode);
};

// Close the socket on destruction, duh.
Udp::~Udp ()
{
    close (socket);
};

} // namespace
