#ifndef DLOWLAT_UDP_SOCKETS_H
#define DLOWLAT_UDP_SOCKETS_H

/******************************************************************************
 * Copyright (C) 2003 - Nelson Posse Lago
 *
 * This file is part of DLADSPA, a system for distributed processing
 * of LADSPA plugins.
 *
 * DLADSPA is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * DLADSPA is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * or just check their website at http://www.fsf.org
 ******************************************************************************/


/******************************************************************************
 *    This file defines some *very* simple classes for UDP communication
 ******************************************************************************/

// Files that include this file may also need to include:
// <netinet/in.h>

// FIXME - There's no code for dealing with errors here.


#include <dlowlat/transport/ip/ipaddress.h>

// fcntl
#include <unistd.h>
#include <fcntl.h>

#include <sys/uio.h> // readv/writev

#include <sys/socket.h> // getsockname

namespace DLowLat {

/******************************************************************************
 * An idiotic base class for UDP communication; this should be seen
 * as a pure virtual class, even if it isn't.
 ******************************************************************************/

class Udp {
public:
    Udp ();
    Udp (Udp&);
    IPAddress address () const;
    virtual ~Udp ();
protected:
    int socket;
private:
    virtual void fillAddress (IPAddress&) const = 0;
};


/******************************************************************************
 * A class for sending UDP data to a remote machine
 ******************************************************************************/

class UdpWrite: public Udp {
public:
    // Specify where to send data to on the constructor
    UdpWrite(IPAddress& address);
    UdpWrite (UdpWrite&);
    inline ssize_t operator () (struct iovec*, int);
private:
    virtual void fillAddress (IPAddress&) const;
};


/******************************************************************************
 * A class for receiving UDP data from a remote machine
 ******************************************************************************/

class UdpRead: public Udp {
public:
    // Specify address/port to bind to on the constructor.
    // If the address is ADDR_ANY, it is modified to correspond to
    // the actual port we binded to.
    UdpRead (IPAddress& address, bool blocking = 1);
    UdpRead (UdpRead&);
    inline ssize_t operator () (struct iovec*, int);
    inline void blockingMode (bool);
    inline bool blockingMode ();
    void flush ();
private:
    void fillAddress (IPAddress&) const;
    bool block;
};


/******************************************************************************
 *                                Inlines
 ******************************************************************************/

//////////// Read and write data ///////////
inline ssize_t
UdpWrite::operator() (struct iovec* data, int size)
{
    return writev (socket, data, size);
};

inline ssize_t
UdpRead::operator () (struct iovec* data, int size)
{
    return readv (socket, data, size);
};

inline void
UdpRead::blockingMode (bool block)
{
    if (block != this->block) {
        if (block) {
            fcntl (socket, F_SETFL, fcntl (socket, F_GETFL) & (~O_NONBLOCK));
        } else {
            fcntl (socket, F_SETFL, fcntl (socket, F_GETFL) | O_NONBLOCK);
        }
        this->block = block;
    }
};

inline bool
UdpRead::blockingMode ()
{
    return block;
};

} // namespace

#endif // DLOWLAT_UDP_SOCKETS_H
