#ifndef DLOWLAT_HOSTNAME_H
#define DLOWLAT_HOSTNAME_H

/******************************************************************************
 * Copyright (C) 2003 - Nelson Posse Lago
 *
 * This file is part of DLADSPA, a system for distributed processing
 * of LADSPA plugins.
 *
 * DLADSPA is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * DLADSPA is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * or just check their website at http://www.fsf.org
 ******************************************************************************/

/******************************************************************************
 * A small wrapper to hostname() that returns a C++-style string instead of
 * a *char buffer.
 ******************************************************************************/


#include <unistd.h>
// FIXME - something weird happens here...
//#include <sys/param.h> // MAXHOSTNAMELEN

namespace DLowLat {

// The linux gethostname manpage talks about HOST_NAME_MAX, but W. Richard
// Stevens, in Unix Net. Programming vol. 1, second edition at page 251
// talks about MAXHOSTNAMELEN; since linux defines MAXHOSTNAMELEN and not
// HOST_NAME_MAX, it's very likely that the manpage is incorrect; still, we
// try to cover every possibility here. As a last resort, we use a large value
// as the maximum and hope for the best.

#ifndef HOST_NAME_MAX
#ifdef MAXHOSTNAMELEN
#define HOST_NAME_MAX MAXHOSTNAMELEN
#else
#define HOST_NAME_MAX 1024
#endif
#endif

inline std::string
getHostname ()
{
    // Find out this machine's hostname
    char hostname[HOST_NAME_MAX +1];
    gethostname (hostname, HOST_NAME_MAX +1);
    return hostname;
};

} // namespace
#endif // DLOWLAT_HOSTNAME_H
