#ifndef DLADSPA_PLUGIN_FACTORY_SERVANT_H
#define DLADSPA_PLUGIN_FACTORY_SERVANT_H

/******************************************************************************
 * Copyright (C) 2003 - Nelson Posse Lago
 *
 * This file is part of DLADSPA, a system for distributed processing
 * of LADSPA plugins.
 *
 * DLADSPA is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * DLADSPA is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * or just check their website at http://www.fsf.org
 ******************************************************************************/


/******************************************************************************
 * This is the servant that creates and destroys processor plugins when
 * requested to do so via CORBA. This version is very tied to the IP
 * transport layer, but it should be easy to create new versions of this
 * servant adequate for other transports, sice it is so simple.
 * When a new plugin is requested, this servant forks a new process and
 * creates the plugin; when a request to destroy the plugin arrives, the
 * servant simply kills the child process. It would be better to use threads,
 * which would mean less context switches, but this works.
 * FIXME: After fork(), we need to close filedescriptors, network connections
 * etc. that are inherited from the parent. Also, the signal handling should
 * be cleaner.
 ******************************************************************************/


#include <list>
#include <utility>
#include <dlowlat/transport/ip/ipaddress.h>

class PluginFactoryImp : public virtual POA_DLADSPA::PluginFactory
{
public:
    IPAddressSpec* newPlugin (const IPAddressSpec& clientAddress,
                    const char* pluginDescriptor, SampleRate rate)
                    throw (CORBA::SystemException);
    void deletePlugin (const IPAddressSpec& serverAddress)
                    throw (CORBA::SystemException);
    virtual ~PluginFactoryImp();
private:
    std::list<std::pair<DLowLat::IPAddress, pthread_t*> > openContainers;
};

#endif
